<!--
LDAP based authentication handler for Leihs 
Copyright 2022 ELAN e.V.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
-->
<!doctype html>
<html>
	<head>
		<title>Login</title>
		<meta name=viewport content="width=device-width, initial-scale=1">
		<link type=text/css rel=stylesheet href={{ url_for('static', filename='style.css') }} />
	</head>
	<body>
		<h1>Login</h1>
		<form action="/" method="POST">
			 <input type="hidden" name="token" value="{{ token }}"/>
			 <input type="text" name="user" value="{{ user }}" readonly disabled />
			 <input type="password" name="password" placeholder="password" autofocus />
			 <input type="submit" value="Login" />
		</form>
	</body>
</html>
